@@ -1,4 +1,4 @@
-/*
+/* 
  * dhcpcd - DHCP client daemon
  * Copyright (c) 2006-2012 Roy Marples <roy@marples.name>
  * All rights reserved
@@ -60,7 +60,6 @@
 #  define _PATH_DEVNULL "/dev/null"
 #endif
 
-static char hostname_buffer[HOSTNAME_MAX_LEN + 1];
 int clock_monotonic;
 static char *lbuf;
 static size_t lbuf_len;
@@ -135,28 +134,6 @@ set_nonblock(int fd)
 	return 0;
 }
 
-const char *
-get_hostname(int short_hostname)
-{
-	char *p;
-
-	gethostname(hostname_buffer, sizeof(hostname_buffer));
-	hostname_buffer[sizeof(hostname_buffer) - 1] = '\0';
-	if (strcmp(hostname_buffer, "(none)") == 0 ||
-	    strcmp(hostname_buffer, "localhost") == 0 ||
-	    strncmp(hostname_buffer, "localhost.", strlen("localhost.")) == 0 ||
-	    hostname_buffer[0] == '.')
-		return NULL;
-
-	if (short_hostname) {
-		p = strchr(hostname_buffer, '.');
-		if (p)
-			*p = '\0';
-	}
-
-	return hostname_buffer;
-}
-
 /* Handy function to get the time.
  * We only care about time advancements, not the actual time itself
  * Which is why we use CLOCK_MONOTONIC, but it is not available on all
@@ -230,11 +207,7 @@ setvar(char ***e, const char *prefix, const char *var, const char *value)
 
 	if (prefix)
 		len += strlen(prefix) + 1;
-	**e = malloc(len);
-	if (**e == NULL) {
-		syslog(LOG_ERR, "%s: %m", __func__);
-		return -1;
-	}
+	**e = xmalloc(len);
 	if (prefix)
 		snprintf(**e, len, "%s_%s=%s", prefix, var, value);
 	else
@@ -277,3 +250,58 @@ writepid(int fd, pid_t pid)
 		return -1;
 	return 0;
 }
+
+#ifndef xmalloc
+void *
+xmalloc(size_t s)
+{
+	void *value = malloc(s);
+
+	if (value != NULL)
+		return value;
+	syslog(LOG_ERR, "memory exhausted (xalloc %zu bytes)", s);
+	exit (EXIT_FAILURE);
+	/* NOTREACHED */
+}
+#endif
+
+#ifndef xrealloc
+void *
+xrealloc(void *ptr, size_t s)
+{
+	void *value = realloc(ptr, s);
+
+	if (value != NULL)
+		return value;
+	syslog(LOG_ERR, "memory exhausted (xrealloc %zu bytes)", s);
+	exit(EXIT_FAILURE);
+	/* NOTREACHED */
+}
+#endif
+
+#ifndef xstrdup
+char *
+xstrdup(const char *str)
+{
+	char *value;
+
+	if (str == NULL)
+		return NULL;
+
+	if ((value = strdup(str)) != NULL)
+		return value;
+
+	syslog(LOG_ERR, "memory exhausted (xstrdup)");
+	exit(EXIT_FAILURE);
+	/* NOTREACHED */
+}
+#endif
+
+void *
+xzalloc(size_t s)
+{
+	void *value = xmalloc(s);
+
+	memset(value, 0, s);
+	return value;
+}
