@@ -37,6 +37,11 @@
 #define UNCONST(a)		((void *)(unsigned long)(const void *)(a))
 
 #define timeval_to_double(tv) ((tv)->tv_sec * 1.0 + (tv)->tv_usec * 1.0e-6)
+#define ms_to_tv(tv, ms) 						\
+	do {								\
+		(tv)->tv_sec = (ms / 1000);				\
+		(tv)->tv_usec = ((ms % 1000) * 1000);			\
+	} while (0 /* CONSTCOND */);
 #define timernorm(tvp)							\
 	do {								\
 		while ((tvp)->tv_usec >= 1000000) {			\
