@@ -573,7 +573,7 @@ add_subnet_route(struct rt *rt, const struct interface *iface)
 }
 
 static struct rt *
-get_routes(const struct interface *iface)
+get_routes(struct interface *iface)
 {
 	struct rt *rt, *nrt = NULL, *r = NULL;
 
@@ -596,8 +596,7 @@ get_routes(const struct interface *iface)
 		return nrt;
 	}
 
-	return get_option_routes(iface->state->new,
-	    iface->name, &iface->state->options->options);
+	return get_option_routes(iface, iface->state->new);
 }
 
 /* Some DHCP servers add set host routes by setting the gateway
@@ -687,7 +686,7 @@ void
 build_routes(void)
 {
 	struct rt *nrs = NULL, *dnr, *or, *rt, *rtn, *rtl, *lrt = NULL;
-	const struct interface *ifp;
+	struct interface *ifp;
 
 	for (ifp = ifaces; ifp; ifp = ifp->next) {
 		if (ifp->state->new == NULL)
