@@ -60,6 +60,11 @@ enum DHS {
 #define LINK_UNKNOWN	0
 #define LINK_DOWN 	-1
 
+#define IF_DATA_DHCP	0
+#define IF_DATA_IPV6RS	1
+#define IF_DATA_DHCP6	2
+#define IF_DATA_MAX	3
+
 struct if_state {
 	enum DHS state;
 	char profile[PROFILE_LEN];
@@ -85,6 +90,7 @@ struct if_state {
 struct interface {
 	char name[IF_NAMESIZE];
 	struct if_state *state;
+	void *if_data[IF_DATA_MAX];
 
 	unsigned int index;
 	int flags;
@@ -111,10 +117,6 @@ struct interface {
 
 	unsigned char *clientid;
 
-	unsigned char *rs;
-	size_t rslen;
-	int rsprobes;
-
 	struct interface *next;
 };
 
