@@ -153,6 +153,8 @@ get_netlink(int fd, int flags,
 	char *buf = NULL, *nbuf;
 	ssize_t buflen = 0, bytes;
 	struct nlmsghdr *nlm;
+	struct sockaddr_nl nladdr;
+	socklen_t nladdr_len = sizeof(nladdr);
 	int r = -1;
 
 	for (;;) {
@@ -181,7 +183,8 @@ get_netlink(int fd, int flags,
 				goto eexit;
 			buf = nbuf;
 		}
-		bytes = recv(fd, buf, buflen, flags);
+		bytes = recvfrom(fd, buf, buflen, flags,
+		    (struct sockaddr *)&nladdr, &nladdr_len);
 		if (bytes == -1) {
 			if (errno == EAGAIN) {
 				r = 0;
@@ -191,6 +194,16 @@ get_netlink(int fd, int flags,
 				continue;
 			goto eexit;
 		}
+
+		/* Check sender */
+		if (nladdr_len != sizeof(nladdr)) {
+			errno = EINVAL;
+			goto eexit;
+		}
+		/* Ignore message if it is not from kernel */
+		if (nladdr.nl_pid != 0)
+			continue;
+
 		for (nlm = (struct nlmsghdr *)(void *)buf;
 		     NLMSG_OK(nlm, (size_t)bytes);
 		     nlm = NLMSG_NEXT(nlm, bytes))
@@ -589,6 +602,7 @@ int
 if_address6(const struct interface *ifp, const struct ipv6_addr *ap, int action)
 {
 	struct nlma *nlm;
+	struct ifa_cacheinfo cinfo;
 	int retval = 0;
 
 	nlm = xzalloc(sizeof(*nlm));
@@ -608,16 +622,46 @@ if_address6(const struct interface *ifp, const struct ipv6_addr *ap, int action)
 	add_attr_l(&nlm->hdr, sizeof(*nlm), IFA_LOCAL,
 	    &ap->addr.s6_addr, sizeof(ap->addr.s6_addr));
 
+	if (action >= 0) {
+		memset(&cinfo, 0, sizeof(cinfo));
+		cinfo.ifa_prefered = ap->prefix_pltime;
+		cinfo.ifa_valid = ap->prefix_vltime;
+		add_attr_l(&nlm->hdr, sizeof(*nlm), IFA_CACHEINFO,
+		    &cinfo, sizeof(cinfo));
+	}
+
 	if (send_netlink(&nlm->hdr) == -1)
 		retval = -1;
 	free(nlm);
 	return retval;
 }
 
+static int
+rta_add_attr_32(struct rtattr *rta, unsigned int maxlen,
+    int type, uint32_t data)
+{
+	unsigned int len = RTA_LENGTH(sizeof(data));
+	struct rtattr *subrta;
+
+	if (RTA_ALIGN(rta->rta_len) + len > maxlen) {
+		errno = ENOBUFS;
+		return -1;
+	}
+
+	subrta = (struct rtattr*)(((char*)rta) + RTA_ALIGN(rta->rta_len));
+	subrta->rta_type = type;
+	subrta->rta_len = len;
+	memcpy(RTA_DATA(subrta), &data, sizeof(data));
+	rta->rta_len = NLMSG_ALIGN(rta->rta_len) + len;
+	return 0;
+}
+
 int
 if_route6(const struct rt6 *rt, int action)
 {
 	struct nlmr *nlm;
+	char metricsbuf[32];
+	struct rtattr *metrics = (void *)metricsbuf;
 	int retval = 0;
 
 	nlm = xzalloc(sizeof(*nlm));
@@ -659,6 +703,14 @@ if_route6(const struct rt6 *rt, int action)
 	add_attr_32(&nlm->hdr, sizeof(*nlm), RTA_OIF, rt->iface->index);
 	add_attr_32(&nlm->hdr, sizeof(*nlm), RTA_PRIORITY, rt->metric);
 
+	if (rt->mtu) {
+		metrics->rta_type = RTA_METRICS;
+		metrics->rta_len = RTA_LENGTH(0);
+		rta_add_attr_32(metrics, sizeof(metricsbuf), RTAX_MTU, rt->mtu);
+		add_attr_l(&nlm->hdr, sizeof(*nlm), RTA_METRICS,
+		    RTA_DATA(metrics), RTA_PAYLOAD(metrics));
+	}
+
 	if (send_netlink(&nlm->hdr) == -1)
 		retval = -1;
 	free(nlm);
