@@ -46,7 +46,8 @@ struct ipv6_addr {
 	uint32_t prefix_vltime;
 	uint32_t prefix_pltime;
 	struct in6_addr addr;
-	int new;
+	uint8_t onlink;
+	uint8_t new;
 	char saddr[INET6_ADDRSTRLEN];
 };
 
@@ -65,6 +66,7 @@ TAILQ_HEAD(rt6head, rt6);
 extern int socket_afnet6;
 
 int ipv6_open(void);
+ssize_t ipv6_printaddr(char *, ssize_t, const uint8_t *, const char *);
 struct in6_addr *ipv6_linklocal(const char *);
 int ipv6_makeaddr(struct in6_addr *, const char *, const struct in6_addr *, int);
 int ipv6_mask(struct in6_addr *, int);
